# -*- coding: utf-8 -*-

import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
plt.rcParams['font.sans-serif'] = ['kaiti']  # 用来正常显示中文标签
plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号

from WindPy import w
w.start()  # 默认命令超时时间为120秒，如需设置超时时间可以加入waitTime参数，例如waitTime=60,即设置命令超时时间为60秒
w.isconnected()  # 判断WindPy是否已经登录成功

index = pd.read_excel('D:/Git/hbshare/hbshare/fe/xwq/data/index_corr/index_1.xlsx')
index['INDEX_SYMBOL'] = index['INDEX_SYMBOL'].astype(str)
index['INDEX_NAME'] = index['INDEX_NAME'].astype(str)
index_list = index['INDEX_SYMBOL'].unique().tolist()[:10]
index_list = ",".join(index_list)
index_dict = index.set_index('INDEX_SYMBOL')['INDEX_NAME'].to_dict()
index_data = pd.read_excel('D:/Git/hbshare/hbshare/fe/xwq/data/index_corr/index_1_data.xlsx')
index_data['TRADE_DATE'] = index_data['TRADE_DATE'].apply(lambda x: x.date().strftime('%Y%m%d'))
index_data = index_data.set_index('TRADE_DATE')

for year in ['2000', '2008', '2015', '2020']:
    start = str(int(year) - 1) + '-12-31'
    end = year + '-12-31'
    index_daily_k = w.wsd(index_list, "close", start, end, usedf=True)[1].reset_index()
    index_daily_k = index_daily_k.rename(columns={'index': 'TRADE_DATE'})
    index_daily_k['TRADE_DATE'] = index_daily_k['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d'))
    index_daily_k = index_daily_k.set_index('TRADE_DATE')
    index_corr = index_daily_k.merge(index_data, left_index=True, right_index=True, how='left')
    index_corr = index_corr.corr()
    index_corr = index_corr.rename(columns=index_dict)
    index_corr = index_corr.rename(index=index_dict)

    plt.figure(figsize=(12, 8))
    sns.heatmap(index_corr, annot=False, fmt='.2f', cmap='coolwarm')
    plt.xlabel('')
    plt.ylabel('')
    plt.title(year, fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
    plt.tight_layout()
    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/index_corr/{0}.png'.format(year))

index = pd.read_excel('D:/Git/hbshare/hbshare/fe/xwq/data/index_corr/index_2.xlsx')
index['INDEX_SYMBOL'] = index['INDEX_SYMBOL'].astype(str)
index['INDEX_NAME'] = index['INDEX_NAME'].astype(str)
index_list = index['INDEX_SYMBOL'].unique().tolist()[:-1]
index_list = ",".join(index_list)
index_dict = index.set_index('INDEX_SYMBOL')['INDEX_NAME'].to_dict()
index_data = pd.read_excel('D:/Git/hbshare/hbshare/fe/xwq/data/index_corr/index_2_data.xlsx')
index_data['TRADE_DATE'] = index_data['TRADE_DATE'].apply(lambda x: x.date().strftime('%Y%m%d'))
index_data['YEAR'] = index_data['TRADE_DATE'].apply(lambda x: str(x)[:4])
index_data = index_data.sort_values('TRADE_DATE').drop_duplicates('YEAR', keep='last')
index_data = index_data.drop('TRADE_DATE', axis=1).set_index('YEAR')

index_daily_k = w.wsd(index_list, "close", "2009-12-31", "2022-10-31", "Period=Y;Days=Alldays", usedf=True)[1].reset_index()
index_daily_k = index_daily_k.rename(columns={'index': 'TRADE_DATE'})
index_daily_k['TRADE_DATE'] = index_daily_k['TRADE_DATE'].apply(lambda x: x.strftime('%Y%m%d'))
index_daily_k['YEAR'] = index_daily_k['TRADE_DATE'].apply(lambda x: str(x)[:4])
index_daily_k = index_daily_k.sort_values('TRADE_DATE').drop_duplicates('YEAR', keep='last')
index_daily_k = index_daily_k.drop('TRADE_DATE', axis=1).set_index('YEAR')

index_ret = index_daily_k.merge(index_data, left_index=True, right_index=True, how='left')
index_ret = index_ret.pct_change().dropna(how='all')
index_ret = index_ret.rename(columns=index_dict)
index_ret = index_ret.T
index_ret.to_excel('D:/Git/hbshare/hbshare/fe/xwq/data/index_corr/index_ret.xlsx')



"""
市值指数
"""
import pandas as pd
import numpy as np
import hbshare as hbs
import os
from tqdm import tqdm
from hbshare.fe.common.util.data_loader import get_trading_day_list
from hbshare.fe.common.util.config import style_name, industry_name
from hbshare.fe.common.util.config import factor_map_dict


class CapIndex:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date

    @staticmethod
    def get_mkt_info(start_date, end_date):
        path = r'D:\kevin\risk_model_jy\RiskModel\data\common_data\chg_pct'
        listdir = os.listdir(path)
        listdir = [x for x in listdir if start_date <= x.split('.')[0] <= end_date]
        data = []
        for filename in listdir:
            date_t_rate = pd.read_csv(os.path.join(path, filename), dtype={"tradeDate": str})
            date_t_rate['ticker'] = date_t_rate['ticker'].apply(lambda x: str(x).zfill(6))
            data.append(date_t_rate)

        data = pd.concat(data)
        # filter
        data = data[(data['dailyReturnReinv'] >= -0.2) & (data['dailyReturnReinv'] <= 0.2)]
        data = pd.pivot_table(data, index='tradeDate', columns='ticker', values='dailyReturnReinv').sort_index()
        data = data.dropna(how='any', axis=1)

        return data

    @staticmethod
    def _load_benchmark_components(date):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and " \
                     "SecuCode in ('000300', '000905', '000852', '000985')"
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code_series = index_info.set_index('SECUCODE')['INNERCODE']

        weight = []
        for benchmark_id in ['000300', '000905', '000852', '000985']:
            inner_code = inner_code_series.loc[benchmark_id]
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "ticker", "ENDDATE": "effDate", "WEIGHT": "weight"})
            weight_df['benchmark_id'] = benchmark_id
            weight.append(weight_df[['ticker', 'benchmark_id']])

        return pd.concat(weight)

    @staticmethod
    def _load_style_exposure(date):
        sql_script = "SELECT * FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}'".format(date)
        res = hbs.db_data_query('alluser', sql_script, page_size=5000)
        exposure_df = pd.DataFrame(res['data']).set_index('ticker')
        ind_names = [x.lower() for x in industry_name['sw'].values()]
        exposure_df = exposure_df[style_name + ind_names]

        return exposure_df

    def calculate_cap_index(self):
        month_list = get_trading_day_list(self.start_date, self.end_date, frequency="month")
        month_list.append(self.end_date)
        reverse_ind = dict([(value.lower(), key) for (key, value) in industry_name['sw'].items()])
        ret_list = []
        style_dict = {}
        industry_dict = {}
        ret_attr = {}
        for i in tqdm(range(1, len(month_list))):
            p_date, t_date = month_list[i - 1], month_list[i]
            period_data = self.get_mkt_info(p_date, t_date).fillna(0.)
            bm_components = self._load_benchmark_components(p_date)
            mkv_info = pd.read_json(
                r'D:\kevin\risk_model_jy\RiskModel\data\common_data/market_value/%s.json' % p_date,
                dtype={'ticker': str})
            mkv_info = mkv_info[(mkv_info['ticker'].str[0].isin(['0', '3', '6']))].set_index(
                'ticker')[['marketValue', 'negMarketValue']]
            style_exposure_df = self._load_style_exposure(p_date)

            idx = set(period_data.columns).intersection(set(bm_components['ticker'].unique())).intersection(
                set(mkv_info.index)).intersection(set(style_exposure_df.index))
            # 指数收益
            period_data = period_data.T.reindex(idx).T
            mkv_info = mkv_info.reindex(idx)
            bm_components = bm_components.merge(mkv_info, left_on='ticker', right_index=True).sort_values(
                by=['ticker', 'benchmark_id']).drop_duplicates(subset=['ticker'], keep='first')
            # tmp = bm_components
            bm_components = pd.pivot_table(
                bm_components, index='ticker', columns='benchmark_id', values='negMarketValue').fillna(0.) / 1e+9
            bm_components = (bm_components / bm_components.sum()).reindex(idx)
            index_ret = (1 + period_data).cumprod().dot(bm_components).pct_change().dropna()
            ret_list.append(index_ret)
            # 指数风格
            # style_exposure_df = style_exposure_df.reindex(idx)
            # index_style = bm_components.T.dot(style_exposure_df.astype(float)).T
            #
            # style_df = index_style.loc[style_name]
            # style_df.index = style_df.index.map(factor_map_dict)
            # benchmark_id_map = {"000300": "沪深300", "000905": "中证500", "000852": "中证1000", "000985": "剩余小票"}
            # style_df = style_df.rename(columns=benchmark_id_map)
            # style_dict[p_date] = style_df
            #
            # ind_df = index_style[10:]
            # ind_df.index = [reverse_ind[x] for x in ind_df.index]
            # ind_df = ind_df.rename(columns=benchmark_id_map)
            # industry_dict[p_date] = ind_df
            # # 风格拆解： 风格 + 行业
            # sql_script = "SELECT * FROM st_ashare.r_st_barra_factor_return where " \
            #              "TRADE_DATE > '{}' and TRADE_DATE <= {}".format(p_date, t_date)
            # res = hbs.db_data_query('alluser', sql_script, page_size=5000)
            # factor_return = pd.DataFrame(res['data'])
            # factor_return = pd.pivot_table(
            #     factor_return, index='trade_date', columns='factor_name', values='factor_ret').sort_index()
            # factor_return = (1 + factor_return).prod() - 1
            # factor_return.index = factor_return.index.str.lower()
            # index_style.loc["country"] = 1.
            #
            # idx = set(factor_return.index).intersection(set(index_style.index))
            #
            # f_ret = index_style.reindex(idx).mul(np.array(factor_return.reindex(idx)).reshape(-1, 1))
            # f_ret.loc['spec'] = (1 + index_ret).prod() - 1 - f_ret.sum()
            #
            # ret_attr[t_date] = f_ret

            # mkv_des = tmp.groupby('benchmark_id')['marketValue'].describe()

        ret_df = pd.concat(ret_list).sort_index()
        ret_df.loc[self.start_date] = 0.
        ret_df = ret_df.sort_index()

        # 风格归因的累计
        attr_list = []
        for date in sorted(ret_attr.keys()):
            tmp = ret_attr[date].copy()
            tmp['factor'] = tmp.index
            tmp['trade_date'] = date
            tmp.reset_index(drop=True, inplace=True)
            attr_list.append(tmp)

        attr_df = pd.concat(attr_list)

        # 沪深300的拆解
        sub_attr = []
        for col in ['000300', '000905', '000852', '000985']:
            sub_df = pd.pivot_table(attr_df, index='trade_date', columns='factor', values=col).sort_index()
            index_return = sub_df.sum(axis=1)
            kt = np.log(1 + index_return) / index_return
            r = (1 + index_return).prod() - 1
            k = np.log(1 + r) / r
            cum_attr = sub_df.T.multiply(kt / k).sum(axis=1).to_frame(col)
            sub_attr.append(cum_attr)

        sub_attr = pd.concat(sub_attr, axis=1)
        sub_attr_style = sub_attr.loc[style_name]
        sub_attr_style.index = sub_attr_style.index.map(factor_map_dict)
        ind_name = [x for x in sub_attr.index.tolist() if x not in style_name]
        ind_name = [x for x in ind_name if x not in ['spec', 'country']]
        sub_attr_style.loc['行业'] = sub_attr.loc[ind_name].sum()
        sub_attr_style.loc['国家因子'] = sub_attr.loc['country']
        sub_attr_style.loc['特质性收益'] = sub_attr.loc['spec']

        sub_attr_ind = sub_attr.loc[ind_name]
        sub_attr_ind.index = [reverse_ind[x] for x in sub_attr_ind.index]

        return ret_df


if __name__ == '__main__':
    # CapIndex('20150101', '20221118').calculate_cap_index()
    CapIndex('20211231', '20221130').calculate_cap_index()


from hbshare.quant.CChen.data.data import load_calendar
from hbshare.quant.CChen.cons import fut_sectors
from hbshare.quant.CChen.cta_factor.const import composite_factor
from hbshare.quant.CChen.db_const import sql_write_path_work
import pandas as pd
from datetime import datetime, timedelta


def cta_factor_line(cta_factors, start_date, end_date, table=None, sql_path=None, freq=''):
    if table is None:
        table = 'cta_index'

    if sql_path is None:
        sql_path = sql_write_path_work['daily']

    cal = load_calendar(
        start_date=start_date, end_date=end_date, freq=freq
    )
    data_all = pd.read_sql_query(
        'select TDATE, CLOSE, FACTOR from ' + table
        + ' where TDATE<=' + end_date.strftime('%Y%m%d')
        + ' and TDATE>=' + start_date.strftime('%Y%m%d')
        + ' and FACTOR in ' + str(tuple(cta_factors.keys())).replace(',)', ')')
        + ' order by TDATE',
        sql_path
    )
    data_all['FACTOR'] = data_all['FACTOR'].apply(lambda x: cta_factors[x])
    data_all = data_all.pivot(index='TDATE', columns='FACTOR', values='CLOSE')
    # data_all = data_all / data_all.loc[data_all.index[0], :]

    data = cal.merge(data_all.reset_index().rename(columns={'TDATE': 't_date'}), on='t_date', how='left')
    data = data.set_index(data['t_date'])[list(cta_factors.values())]
    data = data / data.loc[data.index[0], :]
    return data


# 收益拆分
def cta_factor_analysis(cta_factors, sql_path, start_date, end_date):
    codes = pd.read_sql_query(
        'select EXCHANGE, PCODE, CODE from hsjy_fut_info_p', sql_path
    )
    codes['UCODE'] = codes['EXCHANGE'] * 1000 + codes['PCODE']
    codes = codes.drop_duplicates('UCODE')

    sector_df = pd.DataFrame()
    for i in fut_sectors:
        sector_df = sector_df.append(
            pd.DataFrame(
                {
                    'CODE': fut_sectors[i],
                    'SECTOR': i
                }
            )
        )

    sector_comntribution = None
    for i in cta_factors:
        print(i)
        if i in composite_factor:
            factor_data = pd.read_sql_query(
                'select * from cta_factor where TDATE<=' + end_date.strftime('%Y%m%d')
                + ' and TDATE >=' + start_date.strftime('%Y%m%d')
                + ' and FACTOR in ' + str(tuple(composite_factor[i])) + '',
                sql_path
            )
        else:
            factor_data = pd.read_sql_query(
                'select * from cta_factor where TDATE<=' + end_date.strftime('%Y%m%d')
                + ' and TDATE >=' + start_date.strftime('%Y%m%d')
                + ' and FACTOR=\'' + i + '\'',
                sql_path
            )
        factor_data = factor_data.drop_duplicates(subset=['TDATE', 'UCODE', 'FACTOR'])
        factor_data['ret'] = factor_data['WEIGHT'] * factor_data['RETURN'] * factor_data['POS'] / 10000
        # factor_return = factor_data.pivot(index='TDATE', columns='UCODE', values='ret')
        # factor_return = factor_return.mul((factor_return.sum(axis=1) + 1).shift(1).cumprod().fillna(1), axis=0)
        # return_contribute = factor_return.sum(axis=0)
        return_contribute = factor_data.groupby(by='UCODE').sum()[['ret']].reset_index()

        return_contribute = return_contribute.reset_index().merge(
            codes, on='UCODE', how='left'
        ).merge(
            sector_df, on='CODE', how='left'
        )

        rrr = return_contribute.groupby(by='SECTOR').sum()[['ret']].reset_index()
        if sector_comntribution is None:
            sector_comntribution = rrr.rename(columns={'ret': cta_factors[i]})
        else:
            sector_comntribution = sector_comntribution.merge(
                rrr.rename(columns={'ret': cta_factors[i]}), on='SECTOR', how='left'
            )
    return sector_comntribution


if __name__ == '__main__':

    endDate = datetime(2022, 7, 1).date()
    # startDate = datetime(2022, 6, 13).date()
    startDate = endDate - timedelta(days=365 * 2)
    startDate_analysis = datetime(2022, 6, 1).date()

    factors = {
        'tsrev_d1': '短反转',
        'tsmom_d3': '短时序动量',
        'tsmom_d10': '中时序动量',
        'tsmom_d250': '长时序动量',
        'xsmom_d20_q75': '中截面动量',
        'xsmom_d250_q75': '长截面动量',
        'carry_d20_q70': '截面展期',
        'tscarry_d50': '时序展期',
        'tsbasismom_d3': '短基差动量',
        'xssigma_d20_q75': '波动率',
        # 'xsmrchg_d1_q50': '会员持仓'
    }

    data_table = 'cta_index'
    path = sql_write_path_work['daily']

    factor_line = cta_factor_line(
        cta_factors=factors,
        start_date=startDate,
        end_date=endDate,
        table=data_table,
        sql_path=path,
        freq='w'
    )
    factor_analysis = cta_factor_analysis(
        cta_factors=factors,
        sql_path=path,
        start_date=startDate_analysis,
        end_date=endDate
    )

    factor_line.to_excel(endDate.strftime('%Y%m%d') + '_cta_factors.xlsx')
    # factor_analysis.to_excel(endDate.strftime('%Y%m%d') + '_cta_分板块贡献.xlsx')





# -*- coding: utf-8 -*-

from hbshare.fe.asset_allocation.risk_parity_budget_solver import RiskParity, RiskBudget
from hbshare.fe.asset_allocation.data_loader import Loader
from hbshare.fe.common.util.exception import InputParameterError
from hbshare.fe.common.util.logger import logger
from hbshare.fe.common.util.verifier import verify_type
from multiprocessing import Pool
from numpy import linalg as la
import numpy as np
import pandas as pd


def isPD(A):
    try:
        _ = la.cholesky(A)
        return True
    except la.LinAlgError:
        return False

def nearestPD(A):
    B = (A + A.T) / 2.0
    _, s, V = la.svd(B)
    H = np.dot(V.T, np.dot(np.diag(s), V))
    A2 = (B + H) / 2.0
    A3 = (A2 + A2.T) / 2.0
    if isPD(A3):
        return A3
    spacing = np.spacing(la.norm(A))
    I = np.eye(A.shape[0])
    k = 1
    while not isPD(A3):
        mineig = np.min(np.real(la.eigvals(A3)))
        A3 += I * (-mineig * k**2 + spacing)
        k += 1
    return A3

def correct_cov(cov):
    if not isPD(cov):
        cov = nearestPD(cov)
    min_eig = np.min(np.real(np.linalg.eigvals(cov)))
    if min_eig < 0:
        cov -= 10 * min_eig * np.eye(*cov.shape)
    return cov

class RiskParityBudget:
    def __init__(self, asset_type, asset_list, method, start_date, end_date, is_reblance, reblance_type, n, frequency, compute_risk_days,
                 lb_list=None, ub_list=None, total_weight=None, risk_budget_list=None, processes=16):
        self.asset_type = asset_type
        self.asset_list = asset_list
        self.method = method
        self.start_date = start_date
        self.end_date = end_date
        self.is_reblance = is_reblance
        self.reblance_type = reblance_type
        self.n = n
        self.frequency = frequency
        self.compute_risk_days = compute_risk_days
        self.lb_list = [0.0] * len(self.asset_list) if lb_list is None or lb_list == [] else lb_list
        self.ub_list = [1.0] * len(self.asset_list) if ub_list is None or ub_list == [] else ub_list
        self.total_weight = 1.0 if total_weight is None else total_weight
        self.risk_budget_list = [1.0 / len(self.asset_list)] * len(self.asset_list) if risk_budget_list is None or risk_budget_list == [] else risk_budget_list
        self.risk_budget_list = [budget / sum(self.risk_budget_list) for budget in self.risk_budget_list]
        self.processes = processes
        self._verify_input_param()
        self._load()
        self._init_data()

    def _verify_input_param(self):
        verify_type(self.asset_type, 'asset_type', str)
        verify_type(self.asset_list, 'asset_list', list)
        verify_type(self.method, 'method', str)
        verify_type(self.start_date, 'start_date', str)
        verify_type(self.end_date, 'end_date', str)
        verify_type(self.is_reblance, 'is_reblance', bool)
        verify_type(self.reblance_type, 'reblance_type', str)
        verify_type(self.n, 'n', int)
        verify_type(self.frequency, 'frequency', str)
        verify_type(self.compute_risk_days, 'compute_risk_days', int)
        verify_type(self.lb_list, 'lb_list', list)
        verify_type(self.ub_list, 'ub_list', list)
        verify_type(self.total_weight, 'total_weight', float)
        verify_type(self.risk_budget_list, 'risk_budget_list', list)
        verify_type(self.processes, 'processes', int)
        if self.asset_type not in ['index', 'fund']:
            msg = "asset_type not supported, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if len(self.asset_list) == 0:
            msg = "asset_list is empty, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.method not in ['risk_parity', 'risk_budget']:
            msg = "method not supported, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.reblance_type not in ['init_weight', 'init_target']:
            msg = "reblance_type not supported, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.n <= 0:
            msg = "n must be larger than 0, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.frequency not in ['day', 'week', 'month', 'quarter', 'year']:
            msg = "frequency not supported, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.compute_risk_days <= 0:
            msg = "compute_risk_days must be larger than 0, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if len(self.lb_list) != len(self.asset_list) or len(self.ub_list) != len(self.asset_list):
            msg = "lb_list or ub_list must be the same length with asset_list, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if len([i for i in self.lb_list if (i >= 0.0 and i <= 1.0)]) != len(self.asset_list):
            msg = "lb must be between 0 and 1, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if len([i for i in self.ub_list if (i >= 0.0 and i <= 1.0)]) != len(self.asset_list):
            msg = "ub must be between 0 and 1, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if not (self.total_weight >= 0.0 and self.total_weight <= 1.0):
            msg = "total_weight must be between 0 and 1, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if len(self.risk_budget_list) != len(self.asset_list):
            msg = "risk_budget_list must be the same length with asset_list, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if len([i for i in self.risk_budget_list if (i >= 0.0 and i <= 1.0)]) != len(self.asset_list):
            msg = "risk_budget must be between 0 and 1, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.processes <= 0:
            msg = "processes must be larger than 0, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)

    def _load(self):
        self.calendar_df = Loader().read_cal('19900101', self.end_date)
        self.calendar_df = self.calendar_df.rename(columns={'jyrq': 'CALENDAR_DATE', 'sfjj': 'IS_OPEN', 'sfzm': 'IS_WEEK_END', 'sfym': 'IS_MONTH_END'})
        self.calendar_df['CALENDAR_DATE'] = self.calendar_df['CALENDAR_DATE'].astype(str)
        self.calendar_df = self.calendar_df.sort_values('CALENDAR_DATE')
        self.calendar_df['IS_OPEN'] = self.calendar_df['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
        self.calendar_df['IS_WEEK_END'] = self.calendar_df['IS_WEEK_END'].fillna(0).astype(int)
        self.calendar_df['IS_MONTH_END'] = self.calendar_df['IS_MONTH_END'].fillna(0).astype(int)
        self.calendar_df['YEAR_MONTH'] = self.calendar_df['CALENDAR_DATE'].apply(lambda x: x[:6])
        self.calendar_df['MONTH'] = self.calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:6])
        self.calendar_df['MONTH_DAY'] = self.calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:])
        self.calendar_df['IS_QUARTER_END'] = np.where((self.calendar_df['IS_MONTH_END'] == 1) & (self.calendar_df['MONTH'].isin(['03', '06', '09', '12'])), 1, 0)
        self.calendar_df['IS_QUARTER_END'] = self.calendar_df['IS_QUARTER_END'].astype(int)
        self.calendar_df['IS_SEASON_END'] = np.where(self.calendar_df['MONTH_DAY'].isin(['0331', '0630', '0930', '1231']), 1, 0)
        self.calendar_df['IS_SEASON_END'] = self.calendar_df['IS_SEASON_END'].astype(int)
        self.trade_cal = self.calendar_df[self.calendar_df['IS_OPEN'] == 1]
        self.trade_cal['TRADE_DATE'] = self.trade_cal['CALENDAR_DATE']
        self.calendar_df = self.calendar_df.merge(self.trade_cal[['CALENDAR_DATE', 'TRADE_DATE']], on=['CALENDAR_DATE'], how='left')
        self.calendar_df['TRADE_DATE'] = self.calendar_df['TRADE_DATE'].fillna(method='ffill')
        self.calendar_df = self.calendar_df[['CALENDAR_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END', 'IS_QUARTER_END', 'IS_SEASON_END', 'TRADE_DATE', 'YEAR_MONTH', 'MONTH', 'MONTH_DAY']]

        self.window = self.compute_risk_days
        self.start_date_backup = self.trade_cal[self.trade_cal['TRADE_DATE'] <= self.start_date]['TRADE_DATE'].unique().tolist()[-self.window - 40]

        if self.asset_type == 'index':
            self.mutual_index_nav_df = Loader().read_mutual_index_daily_k_given_indexs(self.asset_list, self.start_date_backup, self.end_date)
            self.mutual_index_nav_df = self.mutual_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(self.mutual_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
            self.mutual_index_nav_df = self.mutual_index_nav_df.drop_duplicates()
            self.mutual_index_nav_df['TRADE_DATE'] = self.mutual_index_nav_df['TRADE_DATE'].astype(str)
            self.mutual_index_nav_df = self.mutual_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
            self.mutual_index_nav_df = self.mutual_index_nav_df.sort_index()

            self.private_index_nav_df = Loader().read_private_index_daily_k_given_indexs(self.asset_list, self.start_date_backup, self.end_date)
            self.private_index_nav_df = self.private_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(self.private_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
            self.private_index_nav_df = self.private_index_nav_df.drop_duplicates()
            self.private_index_nav_df['TRADE_DATE'] = self.private_index_nav_df['TRADE_DATE'].astype(str)
            self.private_index_nav_df = self.private_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
            self.private_index_nav_df = self.private_index_nav_df.sort_index()

            self.market_index_nav_df = Loader().read_market_index_daily_k_given_indexs(self.asset_list, self.start_date_backup, self.end_date)
            self.market_index_nav_df = self.market_index_nav_df[['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']] if len(self.market_index_nav_df) != 0 else pd.DataFrame(columns=['INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX'])
            self.market_index_nav_df = self.market_index_nav_df.drop_duplicates()
            self.market_index_nav_df['TRADE_DATE'] = self.market_index_nav_df['TRADE_DATE'].astype(str)
            self.market_index_nav_df = self.market_index_nav_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
            self.market_index_nav_df = self.market_index_nav_df.sort_index()

            self.nav_df = pd.concat([self.mutual_index_nav_df, self.private_index_nav_df, self.market_index_nav_df], axis=1)

        if self.asset_type == 'fund':
            self.mutual_fund_nav_df = Loader().read_mutual_fund_cumret_given_codes(self.asset_list, self.start_date_backup, self.end_date)
            self.mutual_fund_nav_df = self.mutual_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(self.mutual_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
            self.mutual_fund_nav_df = self.mutual_fund_nav_df.drop_duplicates()
            self.mutual_fund_nav_df['TRADE_DATE'] = self.mutual_fund_nav_df['TRADE_DATE'].astype(str)
            self.mutual_fund_nav_df = self.mutual_fund_nav_df.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
            self.mutual_fund_nav_df = self.mutual_fund_nav_df.sort_index()

            self.private_fund_nav_df = Loader().read_private_fund_adj_nav_given_codes(self.asset_list, self.start_date_backup, self.end_date)
            self.private_fund_nav_df = self.private_fund_nav_df[['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV']] if len(self.private_fund_nav_df) != 0 else pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
            self.private_fund_nav_df = self.private_fund_nav_df.drop_duplicates()
            self.private_fund_nav_df['TRADE_DATE'] = self.private_fund_nav_df['TRADE_DATE'].astype(str)
            self.private_fund_nav_df = self.private_fund_nav_df.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
            self.private_fund_nav_df = self.private_fund_nav_df.sort_index()

            self.nav_df = pd.concat([self.mutual_fund_nav_df, self.private_fund_nav_df], axis=1)

        nan_nav_list = [asset for asset in self.asset_list if asset not in self.nav_df.columns]
        if len(nan_nav_list) == 0:
            self.nav_df = self.nav_df[self.asset_list].sort_index()
        else:
            msg = "{0} no nav data".format(','.join(nan_nav_list))
            logger.error(msg)
            raise InputParameterError(message=msg)
        return

    def _init_data(self):
        # 确定再平衡时点
        if len(self.trade_cal[(self.trade_cal['TRADE_DATE'] >= self.start_date) & (self.trade_cal['TRADE_DATE'] <= self.end_date)]) == 0:
            msg = "no reblance dates between start_date and end_date, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        else:
            self.start_trade_date = self.trade_cal[self.trade_cal['TRADE_DATE'] >= self.start_date]['TRADE_DATE'].values[0]
            if not self.is_reblance:
                reblance_list = [self.start_trade_date]
            else:
                if self.frequency == 'day':
                    reblance_list = self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['TRADE_DATE'].unique().tolist()
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date) and (date <= self.end_date)]
                    reblance_list = reblance_list[::self.n]
                elif self.frequency == 'week':
                    reblance_list = self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['TRADE_DATE'].unique().tolist()
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date) and (date <= self.end_date)]
                    reblance_list = reblance_list[::self.n * 5]
                elif self.frequency == 'month':
                    reblance_list = self.calendar_df['YEAR_MONTH'].unique().tolist()
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date[:6]) and (date <= self.end_date[:6])]
                    reblance_list = reblance_list[::self.n]
                    reblance_list = [date + self.start_trade_date[6:] for date in reblance_list]
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date) and (date <= self.end_date)]
                    reblance_list = [self.calendar_df[(self.calendar_df['IS_OPEN'] == 1) & (self.calendar_df['CALENDAR_DATE'] <= date)]['TRADE_DATE'].iloc[-1] for date in reblance_list]
                elif self.frequency == 'quarter':
                    reblance_list = self.calendar_df['YEAR_MONTH'].unique().tolist()
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date[:6]) and (date <= self.end_date[:6])]
                    reblance_list = reblance_list[::self.n * 3]
                    reblance_list = [date + self.start_trade_date[6:] for date in reblance_list]
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date) and (date <= self.end_date)]
                    reblance_list = [self.calendar_df[(self.calendar_df['IS_OPEN'] == 1) & (self.calendar_df['CALENDAR_DATE'] <= date)]['TRADE_DATE'].iloc[-1] for date in reblance_list]
                elif self.frequency == 'year':
                    reblance_list = self.calendar_df['YEAR_MONTH'].unique().tolist()
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date[:6]) and (date <= self.end_date[:6])]
                    reblance_list = reblance_list[::self.n * 12]
                    reblance_list = [date + self.start_trade_date[6:] for date in reblance_list]
                    reblance_list = [date for date in reblance_list if (date >= self.start_trade_date) and (date <= self.end_date)]
                    reblance_list = [self.calendar_df[(self.calendar_df['IS_OPEN'] == 1) & (self.calendar_df['CALENDAR_DATE'] <= date)]['TRADE_DATE'].iloc[-1] for date in reblance_list]
                else:
                    reblance_list = [self.start_trade_date]

        # 确定每个再平衡时点所需数据
        trading_day_list = self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['TRADE_DATE'].unique().tolist()
        if self.asset_type == 'index' and len(self.private_index_nav_df) > 0:
            date_list = self.calendar_df[self.calendar_df['IS_WEEK_END'] == 1]['TRADE_DATE'].unique().tolist()
            self.q = 52
        elif self.asset_type == 'fund' and len(self.private_fund_nav_df) > 0:
            date_list = self.calendar_df[self.calendar_df['IS_WEEK_END'] == 1]['TRADE_DATE'].unique().tolist()
            self.q = 52
        else:
            date_list = self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['TRADE_DATE'].unique().tolist()
            self.q = 250
        preprocessed_data = dict()
        self.reason_df = pd.DataFrame(index=reblance_list, columns=['原因'])
        if not (self.nav_df.empty or self.nav_df.reindex(date_list).interpolate().dropna(how='all').empty or self.nav_df.dropna(axis=1, how='all').shape[1] != len(self.asset_list)):
            nav_df = self.nav_df.reindex(date_list).interpolate().dropna(how='all').sort_index()
            ret_df = nav_df.pct_change().dropna(how='all')
            for date in reblance_list:
                interval_data = dict()
                start = [td for td in trading_day_list if td <= date][-self.compute_risk_days]
                end = date
                need_length = len(ret_df[(ret_df.index >= start) & (ret_df.index < end)])
                density = [len(self.nav_df[(self.nav_df.index >= start) & (self.nav_df.index < end)][[asset]].dropna()) / float(need_length) if need_length != 0 else 0.0 for asset in self.asset_list]
                if min(density) < 0.3:
                    density_ser = pd.Series(density)
                    density_ser = density_ser[density_ser < 0.3]
                    self.reason_df.loc[date] = '{0}：{1}当期净值数据不足，无法计算预期风险！'.format(date, ','.join([self.asset_list[i] for i in density_ser.index.tolist()]))
                    interval_data['cov_df'] = pd.DataFrame()
                    preprocessed_data[date] = interval_data
                else:
                    interval_data['cov_df'] = ret_df[(ret_df.index >= start) & (ret_df.index < end)].cov() * self.q
                    interval_data['cov_df'] = correct_cov(interval_data['cov_df'])
                    preprocessed_data[date] = interval_data
        else:
            for date in reblance_list:
                self.reason_df.loc[date] = '{0}：{1}当期净值数据不足，无法计算预期风险！'.format(date, ','.join(self.asset_list))
                interval_data = dict()
                interval_data['cov_df'] = pd.DataFrame()
                preprocessed_data[date] = interval_data
        self.preprocessed_data = preprocessed_data
        # cov_df_list = []
        # for date in sorted(list(self.preprocessed_data.keys())):
        #     cov_df_list.append(self.preprocessed_data[date]['cov_df'])
        # all_cov_df = pd.concat(cov_df_list)
        # if len(all_cov_df) == 0:
        #     print('no enough nav data for computing risk model!')
        return

    def get_optimal_one_process(self, date, period_data):
        if period_data['cov_df'].empty:
            weight = pd.DataFrame(index=[date], columns=self.asset_list + ['cash'])
            status = pd.DataFrame(index=[date], columns=['优化状态'], data=['infeasible'])
            status = status.merge(self.reason_df, left_index=True, right_index=True, how='left')
        else:
            try:
                if self.method == 'risk_parity':
                    self.cov = period_data['cov_df']
                    weight, status = RiskParity(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight).solve()
                    weight = pd.DataFrame(weight, columns=[date]).T
                    reason = '{0}：模型当期无优化解！'.format(date) if status == 'infeasible' else np.nan
                    status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
                else:
                    self.cov = period_data['cov_df']
                    weight, status = RiskBudget(self.asset_list, self.cov, self.lb_list, self.ub_list, self.total_weight, self.risk_budget_list).solve()
                    weight = pd.DataFrame(weight, columns=[date]).T
                    reason = '{0}：模型当期无优化解！'.format(date) if status == 'infeasible' else np.nan
                    status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[[status, reason]])
            except:
                weight = pd.DataFrame(index=[date], columns=self.asset_list + ['cash'])
                status = pd.DataFrame(index=[date], columns=['优化状态', '原因'], data=[['infeasible', '{0}：模型当期无优化解！'.format(date)]])
        print('[{0}]/[{1}] done'.format(date, self.method))
        return weight, status

    def get_all(self):
        result_list = []
        p = Pool(processes=self.processes)
        for date, period_data in self.preprocessed_data.items():
            result = p.apply_async(self.get_optimal_one_process, args=(date, period_data))
            result_list.append(result)
        p.close()
        p.join()
        print('processes done.')
        weight_list, status_list = [], []
        for result in result_list:
            weight, status = result.get()
            weight_list.append(weight)
            status_list.append(status)
        weight_df = pd.concat(weight_list).sort_index()
        status_df = pd.concat(status_list).sort_index()
        # 权重归一化（本版本不支持有现金配置）
        weight_df = weight_df[self.asset_list]
        weight_df['TOTAL'] = weight_df.sum(axis=1)
        weight_df['TOTAL'] = weight_df['TOTAL'].replace(0.0, np.nan)
        weight_df = weight_df.apply(lambda x: x.iloc[:len(self.asset_list)] / x[-1], axis=1)
        if self.reblance_type == 'init_weight' and len(weight_df.dropna()) != 0:
            start_index = weight_df.dropna().index[0]
            weight_df.loc[weight_df.index > start_index, :] = np.nan
            weight_df = weight_df.fillna(method='ffill')
            status_df.loc[status_df.index > start_index, :] = np.nan
            status_df = status_df.fillna(method='ffill')
        # if len(weight_df.dropna()) == 0:
        #     print('no optimal results!')
        return weight_df, status_df


if __name__ == '__main__':
    # mutual_index: ['HM0001', 'HM0024', 'HM0095']
    # private_index: ['HB0011', 'HB0016', 'HB1001']
    # market_index: ['000300', '000906', 'CBA00301']
    # mutual_fund: ['002943', '688888', '000729']
    # private_fund: ['SGK768', 'SX8958', 'SR4480']

    weight_df, status_df = RiskParityBudget(asset_type='index',  # index, fund
                                            asset_list=['000300', '000906', 'CBA00301'],
                                            method='risk_parity',  # risk_parity, risk_budget
                                            start_date='20181231',
                                            end_date='20220704',
                                            is_reblance=True,   # True, False
                                            reblance_type='init_target',  # init_weight, init_target
                                            n=3,
                                            frequency='month',  # day, week, month, quarter, year
                                            compute_risk_days=120,  # 交易日
                                            processes=4
                                            ).get_all()
    print(weight_df, status_df)


from hbshare.quant.CChen.data.wind_data import WindApi
import pandas as pd
from datetime import datetime, timedelta
from hbshare.quant.CChen.db_const import sql_write_path_hb, sql_user_hb, sql_write_path_work, sql_user_work

load_end_date = datetime.now() - timedelta(days=1)

daily_path = sql_write_path_hb['daily']
sql_user = sql_user_hb

t1 = datetime.now()
wind_api = WindApi()
sector_df = pd.read_csv('fut_sec_wind.csv')
fut_index_df = pd.read_csv('fut_index_wind.csv')
edb_list = pd.read_csv('wind_edb.csv')
codes_df = pd.DataFrame()
for i in range(len(sector_df)):
    sector = wind_api.api.wset(
        "sectorconstituent",
        "date=" + datetime.now().strftime('%Y-%m-%d')
        + ";sectorid=" + str(sector_df['id'][i]) + ";field=wind_code,sec_name"
    )
    data_df = pd.DataFrame({'code': sector.Data[0]})
    codes_df = codes_df.append(data_df)
codes_df['product'] = codes_df['code'].apply(
    lambda x: x.replace('FI.WI', '') if 'AP' not in x.upper() else x.replace('LFI.WI', '')
)
fut_index_df = fut_index_df.append(codes_df, sort=False)
fut_index_df = fut_index_df.reset_index(drop=True)

# Wind 股指期货日线行情
wind_api.to_sql_stk_index_fut(
    db_path=daily_path,
    table='futures_wind',
    sql_info=sql_user,
    end_date=load_end_date,
    database='daily_data'
)

# Wind 南华指数日线行情
wind_api.to_sql_fut_index(
    db_path=daily_path,
    table='fut_index_wind',
    sql_info=sql_user,
    end_date=load_end_date,
    database='daily_data',
    code_df=pd.read_csv('nh_index_wind.csv')
)

# Wind 万得商品指数日线行情
wind_api.to_sql_fut_index(
    db_path=daily_path,
    table='fut_index_wind',
    sql_info=sql_user,
    end_date=load_end_date,
    database='daily_data',
    code_df=fut_index_df,
)


print(datetime.now() - t1)



"""
风格拥挤度跟踪模块
"""
import pandas as pd
import hbshare as hbs
from datetime import datetime
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params
import pyecharts.options as opts
from pyecharts.charts import Line, Bar
from pyecharts.globals import ThemeType


class FactorCrowd:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date
        self._load_data()

    def _load_data(self):
        engine = create_engine(engine_params)
        # crowd ratio
        sql_script = "SELECT * FROM crowd_ratio where trade_date >= {} and trade_date <= {}".format(
            self.start_date, self.end_date)
        crowd_ratio = pd.read_sql(sql_script, engine)
        crowd_ratio['trade_date'] = crowd_ratio['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        crowd_ratio['mean_ratio'] = crowd_ratio[['mean_to', 'stock_vol', 'beta']].mean(axis=1)
        crowd_ratio.loc[crowd_ratio['factor_name'] == "growth", 'mean_ratio'] = crowd_ratio['beta']
        self.crowd_ratio = pd.pivot_table(
            crowd_ratio, index='trade_date', columns='factor_name', values='mean_ratio').sort_index()
        # ls return
        sql_script = "SELECT * FROM ls_return where trade_date >= {} and trade_date <= {}".format(
            self.start_date, self.end_date)
        ls_return = pd.read_sql(sql_script, engine)
        ls_return['trade_date'] = ls_return['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        self.factor_return = pd.pivot_table(
            ls_return, index='trade_date', columns='factor_name', values='ls_return').sort_index()

    def run(self, factor_name, standardize=True):
        crowd_ratio = self.crowd_ratio.copy()
        if standardize:
            crowd_ratio = (crowd_ratio - crowd_ratio.mean()) / crowd_ratio.std()
        factor_return = self.factor_return.copy()

        if factor_name in crowd_ratio.columns:
            df = crowd_ratio[[factor_name]].merge(
                factor_return[[factor_name]], left_index=True, right_index=True, suffixes=('_crowd', '_ret'))
            df[factor_name + '_nav'] = (1 + df[factor_name + '_ret']).cumprod()
            df = df.round(3)
            # plot
            bar = (
                Bar(init_opts=opts.InitOpts(width='1200px', height='600px'))
                .add_xaxis(
                    xaxis_data=df.index.tolist())
                .extend_axis(
                    yaxis=opts.AxisOpts(
                        type_="value",
                        # name="多空净值",
                        axistick_opts=opts.AxisTickOpts(is_show=True),
                        splitline_opts=opts.SplitLineOpts(is_show=False))
                )
                .set_global_opts(
                    title_opts=opts.TitleOpts(title="风格拥挤度"),
                    tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
                    legend_opts=opts.LegendOpts(pos_top='5%'),
                    yaxis_opts=opts.AxisOpts(
                        type_="value",
                        # name="拥挤度",
                        axistick_opts=opts.AxisTickOpts(is_show=True),
                        splitline_opts=opts.SplitLineOpts(is_show=True),
                    ),
                    xaxis_opts=opts.AxisOpts(
                        type_="category",
                        # axislabel_opts={"interval": "0", "rotate": 90},
                        axistick_opts=opts.AxisTickOpts(is_show=True)),
                    datazoom_opts=[
                        opts.DataZoomOpts(range_start=0, range_end=100),
                        opts.DataZoomOpts(type_="inside")]
                ).add_yaxis(
                    series_name="风格拥挤度(左轴)",
                    y_axis=df[factor_name + '_crowd'].tolist(),
                    label_opts=opts.LabelOpts(is_show=False),
                    itemstyle_opts=opts.ItemStyleOpts(opacity=0.3),
                    # bar_width="50%",
                    z=0
                )
            )

            line = (
                Line()
                .add_xaxis(
                    xaxis_data=df.index.tolist())
                .add_yaxis(
                    series_name="风格多空净值(右轴)",
                    is_smooth=True,
                    y_axis=df[factor_name + '_nav'].tolist(),
                    is_symbol_show=False,
                    linestyle_opts=opts.LineStyleOpts(width=2),
                    label_opts=opts.LabelOpts(is_show=False),
                    yaxis_index=1
                )
            )

            bar.overlap(line)

            bar.render('D:\\123.html')

        elif factor_name in ["factor_compare", "ret_compare"]:
            if factor_name == "factor_compare":
                df = crowd_ratio[['btop', 'growth']]
                label_name1, label_name2 = "成长拥挤度(左轴)", "价值拥挤度(右轴)"
                title_text = "成长 vs 价值（拥挤度）"
                if standardize:
                    df = (df - df.mean()) / df.std()
            else:
                df = (1 + factor_return).cumprod()[['btop', 'growth']]
                label_name1, label_name2 = "成长多空净值(左轴)", "价值多空净值(右轴)"
                title_text = "成长 vs 价值（收益）"

            df = df.round(3)

            line = (
                Line(init_opts=opts.InitOpts(width='1200px', height='600px'))
                .add_xaxis(
                    xaxis_data=df.index.tolist())
                .extend_axis(
                    yaxis=opts.AxisOpts(
                        type_="value",
                        axistick_opts=opts.AxisTickOpts(is_show=True),
                        splitline_opts=opts.SplitLineOpts(is_show=False))
                ).set_global_opts(
                    title_opts=opts.TitleOpts(title=title_text),
                    tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
                    legend_opts=opts.LegendOpts(pos_top='5%'),
                    yaxis_opts=opts.AxisOpts(
                        type_="value",
                        axistick_opts=opts.AxisTickOpts(is_show=True),
                        splitline_opts=opts.SplitLineOpts(is_show=True),
                    ),
                    xaxis_opts=opts.AxisOpts(
                        type_="category",
                        axistick_opts=opts.AxisTickOpts(is_show=True)),
                    datazoom_opts=[
                        opts.DataZoomOpts(range_start=0, range_end=100),
                        opts.DataZoomOpts(type_="inside")]
                ).add_yaxis(
                    series_name=label_name1,
                    is_smooth=True,
                    y_axis=df['growth'].tolist(),
                    is_symbol_show=False,
                    linestyle_opts=opts.LineStyleOpts(width=2),
                    label_opts=opts.LabelOpts(is_show=False),
                    z=0)
            )

            line_sub = (
                Line()
                .add_xaxis(
                    xaxis_data=df.index.tolist())
                .add_yaxis(
                    series_name=label_name2,
                    is_smooth=True,
                    y_axis=df['btop'].tolist(),
                    is_symbol_show=False,
                    linestyle_opts=opts.LineStyleOpts(width=2),
                    label_opts=opts.LabelOpts(is_show=False),
                    yaxis_index=1
                )
            )

            line.overlap(line_sub)

            line.render("D:\\123.html")


if __name__ == '__main__':
    FactorCrowd("20101015", "20221014").run('ret_compare', standardize=False)


import numpy as np
import pandas as pd
from hbshare.fe.XZ import db_engine as dbeng
from hbshare.fe.XZ import functionality


util=functionality.Untils()
hbdb=dbeng.HBDB()
localdb = dbeng.PrvFunDB().engine

def save_mutual_core_pool2locabdb(asofdate):

    data=pd.read_excel(r"C:\Users\xuhuai.zhe\Desktop\核心池列表-20220629_风格.xlsx")

    data=data[['基金代码','基金名称']]
    data['基金经理']=''
    data['asofdate']=asofdate

    data['基金代码']=data['基金代码'].astype(str).str[0:6]

    data.to_sql('core_pool_history',
                con=localdb,index=False,if_exists='append')

def save_wenjian_pool2locabdb(asofdate,factor_name):

    pool_size = 10

    raw_df=pd.read_sql("select * from factor_year_rank_quantile",
                       con=localdb)


    localdb.execute("delete from conservative_pool_{1}_history where asofdate='{0}'".format(asofdate,factor_name.split('%')[0]))

    raw_df=raw_df[(raw_df['date']==asofdate)&(raw_df['count']==17)].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('conservative_pool_{0}_history'.format(factor_name.split('%')[0]),con=localdb,index=False,if_exists='append')

def save_wenjian_pool2locabdb(asofdate,factor_name):

    pool_size = 10

    raw_df=pd.read_sql("select * from factor_year_rank_quantile",
                       con=localdb)


    localdb.execute("delete from conservative_pool_{1}_history where asofdate='{0}'".format(asofdate,factor_name.split('%')[0]))

    raw_df=raw_df[(raw_df['date']==asofdate)&(raw_df['count']==17)].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('conservative_pool_{0}_history'.format(factor_name.split('%')[0]),con=localdb,index=False,if_exists='append')

def save_momentum_pool2locabdb(asofdate):

    pool_size = 20
    factor_name='year_logret'
    fre='Q'
    raw_df=pd.read_sql("select * from factor_year_ret where  {0} is not null "
                       .format(factor_name)
                       ,con=localdb)


    localdb.execute("delete from momentum_pool_history where asofdate='{}'".format(asofdate))

    raw_df=raw_df[(raw_df['date']==asofdate)].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('momentum_pool_history',con=localdb,index=False,if_exists='append')

def save_info_ratio_pool2locabdb(asofdate):

    pool_size = 50
    factor_name='info_ratio'
    fre='Q'
    raw_df=pd.read_sql("select * from factor_infor_ratio where zblb='2101'",
                       con=localdb)


    localdb.execute("delete from info_ratio_pool_history where asofdate='{}'".format(asofdate))

    raw_df=raw_df[(raw_df['date']==int(asofdate))].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('info_ratio_pool_history',con=localdb,index=False,if_exists='append')

if __name__ == '__main__':


    # save_info_ratio_pool2locabdb('20211227')
    # save_info_ratio_pool2locabdb('20220328')
    # save_info_ratio_pool2locabdb('20220627')

    #save_momentum_pool2locabdb('20220630')
    #
    # save_wenjian_pool2locabdb('20220630','25%')
    # save_wenjian_pool2locabdb('20220331','25%')
    # save_wenjian_pool2locabdb('20211231','25%')
    #
    # save_wenjian_pool2locabdb('20220630','10%')
    # save_wenjian_pool2locabdb('20220331','10%')
    # save_wenjian_pool2locabdb('20211231','10%')

    #save_mutual_core_pool2locabdb('202206')

    last_2years = '20191231'


    #ind cons
    sql="SELECT jjdm,max(`占持仓比例(时序均值)`)as max_w from jjpic_industry_detail_1 GROUP BY jjdm "
    d1=pd.read_sql(sql,con=localdb)

    old_jjdm_list=d1['jjdm'].unique().tolist()

    ticker_con = "'" + "','".join(old_jjdm_list) + "'"

    sql = "select jjdm,min(gptzzjb) from st_fund.t_st_gm_zcpz where jsrq<='{0}' and jsrq>={1} and jjdm in ({2}) group by jjdm" \
        .format('20211231', '20200630', ticker_con)

    tempdf = hbdb.db2df(sql, db='funduser')
    old_jjdm_list = tempdf[tempdf['min(gptzzjb)'] >= 60]['jjdm'].tolist()


    d1=d1[d1['max_w']<=0.6]

    sql="SELECT * from jjpic_theme_p"
    d2=pd.read_sql(sql,con=localdb)
    d2['max_w'] = d2[['大金融', '消费', 'TMT', '周期', '制造']].max(axis=1)
    d2 = d2[d2['max_w'] <= 0.7]

    #size cons
    sql="select jjdm,jjjzc from st_fund.t_st_gm_zcpz where jsrq='20211231' and jjdm in ({})"\
        .format(util.list_sql_condition(old_jjdm_list))

    d3=hbdb.db2df(sql,db='funduser')
    d3=d3[(d3['jjjzc']<=50000000000)]

    #manager length
    sql="select jjdm,rydm from st_fund.t_st_gm_jjjl where jjdm in ({}) and ryzw='基金经理' and lrrq>='20211231'"\
        .format(util.list_sql_condition(old_jjdm_list))
    d4=hbdb.db2df(sql,db='funduser')
    sql="select distinct(rydm) from st_fund.t_st_gm_jjjl where rydm in ({0}) and rzrq<='20161231' "\
        .format(util.list_sql_condition(d4['rydm'].astype(str).unique().tolist()))
    d44=hbdb.db2df(sql,db='funduser')
    d44['flag']=1
    d4=pd.merge(d4,d44,how='left',on='rydm')
    d4=d4[d4['flag'].notnull()]


    d=pd.merge(d1,d2,how='inner',on='jjdm')
    d=pd.merge(d,d3,how='inner',on='jjdm')
    d=pd.merge(d,d4,how='inner',on='jjdm')
    new_jjdm_list=d['jjdm'].unique().tolist()

    plot=functionality.Plot(600,600)
    sql="SELECT avg(`成长绝对暴露(持仓)`) as `成长持仓占比`, avg(`价值绝对暴露(持仓)`) as `价值持仓占比` from jjpic_value_p_hbs  "
    pool_value_pic=pd.read_sql(sql,con=localdb)
    plot.plotly_pie(pool_value_pic.T.rename(columns={0:'weight'}),'成长价值占比',save_local_file=True)

    sql="SELECT `成长绝对暴露(持仓)` as `成长持仓占比`, `价值绝对暴露(持仓)` as `价值持仓占比` from jjpic_value_p_hbs  "
    pool_value_pic=pd.read_sql(sql,con=localdb)
    plot.plotly_hist(pool_value_pic[['成长持仓占比']],'成长持仓占比',save_local_file=True)
    plot.plotly_hist(pool_value_pic[['价值持仓占比']],'价值持仓占比',save_local_file=True)


    sql="SELECT avg(`大盘绝对暴露(持仓)`) as `大盘持仓占比`, avg(`中盘绝对暴露(持仓)`) as `中盘持仓占比`,avg(`小盘绝对暴露(持仓)`) as `小盘持仓占比` from jjpic_size_p_hbs  "
    pool_value_pic=pd.read_sql(sql,con=localdb)
    plot.plotly_pie(pool_value_pic.T.rename(columns={0:'weight'}),'规模占比分布',save_local_file=True)

    sql="SELECT `大盘绝对暴露(持仓)` as `大盘持仓占比`, (`中盘绝对暴露(持仓)`+`小盘绝对暴露(持仓)`) as `中小盘持仓占比` from jjpic_size_p_hbs  "
    pool_value_pic=pd.read_sql(sql,con=localdb)
    plot.plotly_hist(pool_value_pic[['大盘持仓占比']],'大盘持仓占比',save_local_file=True)
    plot.plotly_hist(pool_value_pic[['中小盘持仓占比']],'中小盘持仓占比',save_local_file=True)


    sql="SELECT avg(`大金融`) as `大金融`, avg(`消费`) as `消费`,avg(TMT) as TMT,avg(`制造`) as `制造`,avg(`周期`) as `周期`  from jjpic_theme_p   "
    pool_theme_pic=pd.read_sql(sql,con=localdb)
    plot.plotly_pie(pool_theme_pic.T.rename(columns={0:'weight'}),'主题占比分布',save_local_file=True)

    edf=pd.read_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\均衡池_20211231.xlsx")
    edf2=pd.read_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\均衡池_20220331.xlsx")
    #
    # sql="select avg(hb1y) as bmk_old,tjyf from st_fund.t_st_gm_yhb where jjdm in ({}) and tjyf>='201612' and tjyf<='202206' and hb1y!=99999 group by tjyf"\
    #     .format(util.list_sql_condition(old_jjdm_list))
    # ret1=hbdb.db2df(sql,db='funduser')

    sql="select avg(hb1y) as bmk ,tjyf from st_fund.t_st_gm_yhb where jjdm in ({}) and tjyf>='201812' and tjyf<='202206' and hb1y!=99999 group by tjyf"\
        .format(util.list_sql_condition(new_jjdm_list))
    ret2=hbdb.db2df(sql,db='funduser')

    sql="select hb1y as 偏股混指数 ,tjyf from st_market.t_st_zs_yhb where zqdm='885001' and  tjyf>'202112' and tjyf<='202206'"
    ret=hbdb.db2df(sql,db='alluser')

    sql = "select avg(hb1y) as eq_ret ,tjyf from st_fund.t_st_gm_yhb where jjdm in ({}) and tjyf>'202112' and tjyf<='202203' and hb1y!=99999 group by tjyf" \
        .format(util.list_sql_condition(edf['jjdm'].tolist()))
    ret2 = hbdb.db2df(sql, db='funduser')
    sql = "select avg(hb1y) as eq_ret ,tjyf from st_fund.t_st_gm_yhb where jjdm in ({}) and tjyf>'202203' and tjyf<='202206' and hb1y!=99999 group by tjyf" \
        .format(util.list_sql_condition(edf2['jjdm'].tolist()))
    ret3 = hbdb.db2df(sql, db='funduser')

    ret2=pd.concat([ret2,ret3],axis=0)

    ret = pd.merge(ret, ret2, how='left', on='tjyf')
    ret['tjyf']=ret['tjyf'].astype(str)
    ret.set_index('tjyf',inplace=True)
    ret=ret/100+1
    for col in ret:

        ret[col]=ret[col].cumprod()

    plot.plotly_line_style(ret,'业绩走势比较',save_local_file=True)

    print('Done')


"""
雪球产品测算
"""
import hbshare as hbs
import pandas as pd
from hbshare.fe.common.util.data_loader import get_trading_day_list


def snow_ball_simulation(start_date, end_date, duration, knock_in_price, knock_out_price, mode="common"):
    """
    start_date: 回测起始时间
    end_date: 回测结束时间
    duration: 合约期限, 以日计算
    knock_in_price: 敲入价格
    knock_out_price: 敲出价格
    mode: common代表普通雪球, worst代表亏损有限型, step_down代表敲出下移型
    """
    sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                 "and JYRQ >= {} and JYRQ <= {}".format('000905', start_date, end_date)
    index_df = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
    index_series = index_df.set_index('TRADEDATE')['TCLOSE']
    # 每个交易日发一只雪球产品
    date_list = get_trading_day_list(start_date, end_date)

    res_df = pd.DataFrame(index=date_list[:-duration], columns=['c1', 'c2', 'c3', 'ko_time', 'lose_ratio'])

    if mode in ["common", "worst"]:
        for i in range(len(date_list) - duration):
            start_dt, end_dt = date_list[i], date_list[i + duration]
            period_data = index_series.loc[start_dt: end_dt]
            period_data = period_data / period_data.iloc[0]
            ko_date = period_data.index.tolist()[::20][3:]

            if period_data.loc[ko_date].gt(knock_out_price).sum() > 0:  # 情形1：敲出获利:
                tmp = period_data.loc[ko_date].to_frame('nav')
                tmp.loc[tmp['nav'] > knock_out_price, 'sign'] = 1
                ko_time = tmp.index.to_list().index(tmp['sign'].first_valid_index()) + 3
                res_df.loc[start_dt, 'c1'] = 1
                res_df.loc[start_dt, 'ko_time'] = ko_time
            elif period_data[1:].min() >= knock_in_price:  # 情形2：既未敲出也未敲入
                res_df.loc[start_dt, 'c2'] = 1
            else:
                res_df.loc[start_dt, 'c3'] = 1
                lose_ratio = \
                    max(0,
                        (period_data.loc[start_dt] - period_data.loc[end_dt]) / period_data.loc[start_dt])
                res_df.loc[start_dt, 'lose_ratio'] = lose_ratio * (-1)
    else:
        price_list = [1.03 - 0.015 * i for i in range(10)]
        for i in range(len(date_list) - duration):
            start_dt, end_dt = date_list[i], date_list[i + duration]
            period_data = index_series.loc[start_dt: end_dt]
            period_data = period_data / period_data.iloc[0]
            ko_date = period_data.index.tolist()[::20][3:]

            price_series = pd.Series(index=ko_date, data=price_list)
            compare_df = period_data.loc[ko_date].to_frame('price').merge(
                price_series.to_frame('ko_price'), left_index=True, right_index=True)
            compare_df.loc[compare_df['price'] > compare_df['ko_price'], 'sign'] = 1.

            if compare_df['sign'].gt(0).sum() > 0:  # 情形1：敲出获利:
                ko_time = compare_df.index.to_list().index(compare_df['sign'].first_valid_index()) + 3
                res_df.loc[start_dt, 'c1'] = 1
                res_df.loc[start_dt, 'ko_time'] = ko_time
            elif period_data[1:].min() >= knock_in_price:  # 情形2：既未敲出也未敲入
                res_df.loc[start_dt, 'c2'] = 1
            else:
                res_df.loc[start_dt, 'c3'] = 1
                lose_ratio = \
                    max(0,
                        (period_data.loc[start_dt] - period_data.loc[end_dt]) / period_data.loc[start_dt])
                res_df.loc[start_dt, 'lose_ratio'] = lose_ratio * (-1)

    if mode == "worst":
        tmp = res_df[res_df['lose_ratio'] < knock_in_price - 1]
        ratio = tmp.shape[0] / res_df.shape[0]
        print("尾部风险概率: {}".format(format(ratio, '.2%')))
        res_df.loc[res_df['lose_ratio'] < knock_in_price - 1, 'lose_ratio'] = knock_in_price - 1
    print("各情景比例: c1: {}, c2: {}, c3: {}".format(
        format(res_df['c1'].sum() / res_df.shape[0], '.2%'), format(res_df['c2'].sum() / res_df.shape[0], '.2%'),
        format(res_df['c3'].sum() / res_df.shape[0], '.2%')))
    print("平均敲出时间(月): {}".format(round(res_df['ko_time'].mean(), 2)))
    print("平均亏损比例: {}".format(format(res_df['lose_ratio'].mean(), '.2%')))

    return res_df


if __name__ == '__main__':
    snow_ball_simulation('20100101', '20220930', 240, 0.8, 1.0, "step_down")



import  numpy as np
import pandas as pd
from hbshare.fe.XZ import  functionality
from hbshare.fe.XZ import db_engine as dbeng
import datetime

util=functionality.Untils()
hbdb=dbeng.HBDB()
localdb=dbeng.PrvFunDB().engine

def get_jjnav_bydate(jjdm_list,date_list):

    sql="select jjdm,jzrq,fqdwjz from st_fund.t_st_gm_rhb where jzrq in ({0}) and jjdm in ({1})"\
        .format(util.list_sql_condition(date_list),
                util.list_sql_condition(jjdm_list))

    jj_nav=hbdb.db2df(sql,db='funduser')

    return  jj_nav

def create_stock_shift_ratio():

    hsl=pd.read_excel(r"E:\GitFolder\hbshare\fe\factor_analysis\换手率.xlsx")
    date_col=hsl.columns.tolist()
    date_col.remove('证券简称')
    date_col.remove('证券代码')
    new_date_col=[]
    date_map=dict()
    hsl['jjdm'] = hsl['证券代码'].str[0:6]
    for i in range(len(date_col)):
        new_date_col.append(date_col[i].replace('H1','0630').replace(' ','').replace('H2', '1231'))

    date_map = dict(zip(date_col,new_date_col))

    outputdf=pd.DataFrame()
    tempdf=pd.DataFrame()
    tempdf['jjdm']=util.get_mutual_stock_funds('20211231')
    tempdf=pd.merge(tempdf,hsl,how='left',on='jjdm')

    for date in date_col:
        tempdf2=tempdf[['jjdm',date]]
        tempdf2['date']=date_map[date]
        tempdf2.rename(columns={date:'hsl'},inplace=True)
        outputdf=pd.concat([outputdf,tempdf2],axis=0)

    outputdf=outputdf[outputdf['hsl'].notnull()]

    outputdf.to_sql('factor_hsl',con=localdb,index=False,if_exists='append')

def read_factors(table_name):

    sql="select * from {}".format(table_name)
    df=pd.read_sql(sql,con=localdb)

    return  df

def bhar(arr):
    return 100 * (np.power(np.cumprod((arr + 100) / 100).tolist()[-1], 1 / len(arr)) - 1)

def get_quarter_ret(arr):

    return  np.cumprod(arr/100+1).tolist()[-1]-1

def hsl_rank2db(dir):

    #get the hsl raw data first
    # sql="select * from factor_hsl  "
    # df=pd.read_sql(sql,con=localdb)
    df=pd.read_csv(dir)
    df['hsl_rank']=df.groupby('date').rank(method='min')
    count=df.groupby('date', as_index=False).count()[['date', 'jjdm']]
    count.rename(columns={'jjdm':'count'},inplace=True)
    df=pd.merge(df,count,how='left',on='date')
    df['hsl_rank']=df['hsl_rank']/df['count']
    df.to_sql('factor_hsl',con=localdb,index=False,if_exists='append')

def save_fund_return_factor2db():

    jjdm_list=util.get_mutual_stock_funds('20211231')

    #get year end date

    sql_script = "SELECT JYRQ, SFJJ, SFYM,SFZM FROM funddb.JYRL WHERE JYRQ >= {0} and JYRQ <= {1} and SFJJ=0 ".format(
        '20141231', datetime.datetime.today().strftime('%Y%m%d'))
    trade_calander = hbdb.db2df(sql_script, db='readonly').drop('ROW_ID',axis=1).sort_values('JYRQ')
    month_end_list=trade_calander[trade_calander['SFYM']=='1']['JYRQ'].tolist()
    #
    # trade_calander['year'] = trade_calander['JYRQ'].astype(str).str[0:4]
    # year_end_list=trade_calander.groupby('year')['JYRQ'].max().tolist()

    #get month end jj fqnav

    jj_nav=get_jjnav_bydate(jjdm_list,month_end_list)
    jj_nav['month'] = jj_nav['jzrq'].astype(str).str[4:6]
    jj_nav['date']=jj_nav['jzrq']
    jj_nav.set_index('jzrq',inplace=True)
    #get jj year return for each month end date
    jj_nav['quarter_logret']=np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(3)+1)
    jj_nav['half_year_logret']=np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(6)+1)
    jj_nav['year_logret']=np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(12)+1)
    jj_nav['2year_logret'] = np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(24) + 1)
    jj_nav['3year_logret'] = np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(36) + 1)

    jj_nav['year_end_flag']=0
    jj_nav['quarter_end_flag'] = 0
    jj_nav.loc[jj_nav['month']=='12','year_end_flag']=1
    jj_nav.loc[jj_nav['month'].isin(['03','06','09','12'])
    , 'quarter_end_flag'] = 1
    jj_nav['date']=jj_nav['date'].astype(str)

    #localdb.execute('delete from factor_year_ret')
    jj_nav[jj_nav.notnull()].rename(columns={'jzrq':'date'})[['jjdm','quarter_logret','half_year_logret'
        , 'year_logret','2year_logret','3year_logret', 'date','year_end_flag', 'quarter_end_flag']].to_sql('factor_year_ret'
                                                                       ,con=localdb
                                                                       ,index='False'
                                                                       ,if_exists='append')

def save_fund_rank_factor2db():

    #get the quarter end date
    sql= "SELECT JYRQ, SFYM FROM funddb.JYRL WHERE JYRQ >= {0} and JYRQ<={1} and SFJJ=0 and SFYM=1".format(
        '20160101',datetime.datetime.today().strftime('%Y%m%d'))
    calander = hbdb.db2df(sql, db='readonly').drop('ROW_ID',axis=1)
    calander['month']=calander['JYRQ'].astype(str).str[4:6]
    calander=calander.loc[calander['month'].isin(['03','06','09','12'])]['jsrq'].tolist()
    # calander=calander['JYRQ'].tolist()
    calander.sort()

    result=[[],[],[]]
    result2 = [[], [], []]
    result3=[[],[],[]]
    result4 = [[], [], []]


    for date in calander:
        fund_pool=util.get_mutual_stock_funds(date,2)
        sql = "select jjdm,tjyf,rqzh,hb1y from st_fund.t_st_gm_yhb where jjdm in ({0}) and rq1y!=99999 {1} {2} " \
            .format(util.list_sql_condition(fund_pool)
                    , "and rqzh>='{}'".format( str(int(date[0:4])-5)+date[4:6]+'01')
                    , "and rqzh<='{}'".format(date))
        montly_ret = hbdb.db2df(sql, db='funduser')

        #get quarter,half_yearly,yearly return
        montly_ret['hb1q']=montly_ret.groupby('jjdm')['hb1y'].rolling(3).apply(get_quarter_ret).values
        montly_ret['hb1hy']=montly_ret.groupby('jjdm')['hb1y'].rolling(6).apply(get_quarter_ret).values
        montly_ret['hb1y']=montly_ret.groupby('jjdm')['hb1y'].rolling(12).apply(get_quarter_ret).values
        montly_ret['month']=montly_ret['rqzh'].astype(str).str[4:6]
        first_month=montly_ret['tjyf'].min()
        montly_ret=montly_ret.loc[(montly_ret['month'].isin(['03','06','09','12']))&(montly_ret['tjyf']!=first_month)]


        montly_ret=pd.merge(montly_ret,montly_ret.groupby('tjyf').mean()[['hb1q', 'hb1hy', 'hb1y']].rename(columns={'hb1q':'hb1q_mean'
            , 'hb1hy':'hb1hy_mean'
            , 'hb1y':'hb1y_mean'}),how='left',on='tjyf')

        montly_ret=pd.merge(montly_ret,montly_ret.groupby('tjyf').median()[['hb1q', 'hb1hy', 'hb1y']].rename(columns={'hb1q':'hb1q_median'
            , 'hb1hy':'hb1hy_median'
            , 'hb1y':'hb1y_median'}),how='left',on='tjyf')


        montly_ret=pd.merge(montly_ret
                            ,montly_ret.groupby('tjyf').count()['jjdm'].to_frame('jj_per_month')
                            ,how='left',on='tjyf')

        for col in ['hb1q', 'hb1hy', 'hb1y']:
            #get the winning flag
            montly_ret[col + '_win_mean'] = 0
            montly_ret.loc[montly_ret[col] > montly_ret[col + '_mean']
            , col + '_win_mean'] = 1
            montly_ret[col + '_win_median'] = 0
            montly_ret.loc[montly_ret[col] > montly_ret[col + '_median']
            , col + '_win_median'] = 1

            #get the quarter rank
            montly_ret[col+'_rank']=montly_ret.groupby('tjyf').rank(method='min')['hb1q']
            montly_ret[col + '_rank']=montly_ret[col+'_rank']/montly_ret['jj_per_month']


        sql="select jjdm,ryxm,rydm,rzrq,lrrq,moddt_etl from st_fund.t_st_gm_jjjl where jjdm in ({0}) and rzrq<={1}  and ryzw='基金经理'"\
            .format(util.list_sql_condition(montly_ret['jjdm'].unique().tolist())
                    ,int(date)-10000,date)
        available_manager=hbdb.db2df(sql,db='funduser')
        #date cleaning

        available_manager=available_manager.sort_values('moddt_etl')
        available_manager=available_manager.drop_duplicates(['jjdm','ryxm'],keep='last')

        manager_ret=pd.merge(montly_ret,available_manager,how='left',on='jjdm')
        manager_ret['rz_length'] = manager_ret['rqzh'] - manager_ret['rzrq']
        manager_ret=manager_ret[manager_ret['rz_length']>=10000]
        manager_ret=manager_ret[manager_ret['lrrq']>=manager_ret['rqzh']]

        #get the average rank per manager
        manager_ret2=manager_ret.groupby(['ryxm','tjyf']).mean()[['hb1q_rank', 'hb1hy_rank', 'hb1y_rank']].reset_index()

        tempdf = montly_ret[['jjdm', 'hb1q_rank', 'hb1hy_rank', 'hb1y_rank']].groupby('jjdm').describe()
        tempdf1=montly_ret[['jjdm', 'hb1q_rank', 'hb1hy_rank', 'hb1y_rank']].groupby('jjdm').quantile(0.1)
        tempdf2=montly_ret[['jjdm', 'hb1q_rank', 'hb1hy_rank', 'hb1y_rank']].groupby('jjdm').quantile(0.2)

        tempdf_ma = manager_ret2[['ryxm', 'hb1q_rank', 'hb1hy_rank', 'hb1y_rank']].groupby('ryxm').describe()
        tempdf1_ma=manager_ret2[['ryxm', 'hb1q_rank', 'hb1hy_rank', 'hb1y_rank']].groupby('ryxm').quantile(0.1)
        tempdf2_ma=manager_ret2[['ryxm', 'hb1q_rank', 'hb1hy_rank', 'hb1y_rank']].groupby('ryxm').quantile(0.2)

        for j in range(3):
            col=['hb1q', 'hb1hy', 'hb1y'][j]
            #calculate the winning pro
            out_put = (montly_ret.groupby('jjdm').sum()['{}_win_mean'.format(col)] / montly_ret.groupby('jjdm').count()['{}_win_mean'.format(col)]).to_frame('win_mean')
            out_put['win_median']=montly_ret.groupby('jjdm').sum()['{}_win_median'.format(col)] / montly_ret.groupby('jjdm').count()['{}_win_median'.format(col)]
            out_put['count']=montly_ret.groupby('jjdm').count()['{}_win_median'.format(col)]
            out_put['date']=date
            result[j].append(out_put.reset_index())
            # calculate the winning pro by manager
            out_put = (manager_ret.groupby(['ryxm','jjdm']).sum()['{}_win_mean'.format(col)] / manager_ret.groupby(['ryxm','jjdm']).count()['{}_win_mean'.format(col)]).to_frame('win_mean')
            out_put['win_median']=manager_ret.groupby(['ryxm','jjdm']).sum()['{}_win_median'.format(col)] / manager_ret.groupby(['ryxm','jjdm']).count()['{}_win_median'.format(col)]
            out_put['count']=manager_ret.groupby(['ryxm','jjdm']).count()['hb1y_win_median']
            out_put.reset_index(inplace=True)
            out_put=pd.merge(out_put,manager_ret.groupby(['ryxm']).count()['{}_win_median'.format(col)].to_frame('total_count')
                             ,how='left',on='ryxm')
            for col in ['win_mean','win_median']:
                out_put[col]=out_put[col]*out_put['count']/out_put['total_count']

            out_put['date']=date
            result2[j].append(out_put)

            col = ['hb1q', 'hb1hy', 'hb1y'][j]
            #get the rank quantile
            out_put=tempdf[col+'_rank']
            out_put['10%']=tempdf1[col+'_rank']
            out_put['20%']=tempdf2[col+'_rank']
            result3.append(out_put)

            #get the rank quantile per manager
            out_put=tempdf_ma[col+'_rank']
            out_put['10%']=tempdf1_ma[col+'_rank']
            out_put['20%']=tempdf2_ma[col+'_rank']
            result4.append(out_put)



    name_list = ['quarter', 'half_year', 'year']

    for i in range(3):

        sql="delete from factor_{}_win_pro".format(name_list[i])
        localdb.execute(sql,con=localdb)
        pd.concat(result[i],axis=0).to_sql('factor_{}_win_pro'.format(name_list[i])
                                           ,con=localdb,index=False,if_exists='append')

        sql="delete from factor_manager_{}_win_pro".format(name_list[i])
        localdb.execute(sql,con=localdb)
        pd.concat(result2[i],axis=0).to_sql('factor_manager_{}_win_pro'.format(name_list[i])
                                           ,con=localdb,index=False,if_exists='append')

        sql="delete from factor_{}_distribution".format(name_list[i])
        localdb.execute(sql,con=localdb)
        pd.concat(result[i],axis=0).to_sql('factor_{}_distribution'.format(name_list[i])
                                           ,con=localdb,index=False,if_exists='append')

        sql="delete from factor_manager_{}_distribution".format(name_list[i])
        localdb.execute(sql,con=localdb)
        pd.concat(result2[i],axis=0).to_sql('factor_manager_{}_distribution'.format(name_list[i])
                                           ,con=localdb,index=False,if_exists='append')

def save_fund_size2db():

    sql= "SELECT distinct(jsrq) as jsrq FROM st_fund.t_st_gm_zcpz WHERE jsrq >= {0} and jsrq<={1} ".format(
        '20160101',datetime.datetime.today().strftime('%Y%m%d'))
    calander = hbdb.db2df(sql, db='funduser').sort_values('jsrq')
    calander['month_year']=calander['jsrq'].astype(str).str[0:6]
    calander.drop_duplicates('month_year',keep='last',inplace=True)
    calander['month']=calander['jsrq'].astype(str).str[4:6]
    calander=calander.loc[calander['month'].isin(['03','06','09','12'])]['jsrq'].tolist()
    calander.sort()

    fund_size_df=[]

    for date in calander:
        fund_pool = util.get_mutual_stock_funds(str(date), 2)
        sql="select jjdm,jjzzc,jsrq from st_fund.t_st_gm_zcpz where jsrq='{0}' and jjdm in ({1})"\
            .format(date,util.list_sql_condition(fund_pool))
        fund_size_df.append(hbdb.db2df(sql,db='funduser'))

    fund_size_df=pd.concat(fund_size_df,axis=0).reset_index(drop=True)
    fund_size_df.rename(columns={'jsrq': 'date'}, inplace=True)

    sql = "delete from factor_size"
    localdb.execute(sql, con=localdb)
    fund_size_df.to_sql('factor_size'
                        , con=localdb, index=False, if_exists='append')

def save_informationratio2db(rolling='2106'):

    sql= "SELECT distinct(tjrq) as tjrq from  st_fund.t_st_gm_zqjxxbl where tjrq>=20151231 "
    calander = hbdb.db2df(sql, db='funduser')
    calander['month_year']=calander['tjrq'].astype(str).str[0:6]
    calander.drop_duplicates('month_year',keep='last',inplace=True)
    calander['month']=calander['tjrq'].astype(str).str[4:6]
    calander=calander.loc[calander['month'].isin(['03','06','09','12'])]['tjrq'].tolist()
    # calander=calander['tjrq'].tolist()
    calander.sort()

    data=[]
    for date in calander:
        fund_pool = util.get_potient_mutual_stock_funds(str(date))
        sql="select jjdm,tjrq,zbnp as info_ratio from st_fund.t_st_gm_zqjxxbl where zblb='{2}' and tjrq={0} and jjdm in ({1}) and zbnp!=99999 "\
            .format(int(date),util.list_sql_condition(fund_pool),rolling)
        data.append(hbdb.db2df(sql,db='funduser'))

    data=pd.concat(data,axis=0).reset_index(drop=True)
    data.rename(columns={'tjrq':'date'},inplace=True)
    data['zblb']=rolling

    sql = "delete from factor_infor_ratio where zblb='{}'".format(rolling)
    localdb.execute(sql, con=localdb)
    data.to_sql('factor_infor_ratio', con=localdb, index=False, if_exists='append')

def save_share_change2db():

    sql = "SELECT distinct(jsrq) as tjrq from  st_fund.t_st_gm_fecyrbd where jsrq>=20151231 "
    calander = hbdb.db2df(sql, db='funduser').sort_values('tjrq')
    calander['month_year'] = calander['tjrq'].astype(str).str[0:6]
    calander.drop_duplicates('month_year', keep='last', inplace=True)
    calander['month'] = calander['tjrq'].astype(str).str[4:6]
    calander = calander.loc[calander['month'].isin(['03', '06', '09', '12'])]['tjrq'].tolist()
    calander.sort()

    data = []
    for date in calander:
        fund_pool = util.get_mutual_stock_funds(date, 2)
        sql = "select jjdm,jsrq,jgcybl,cyrs  from st_fund.t_st_gm_fecyrbd where jsrq='{0}' and jjdm in ({1}) " \
            .format(date, util.list_sql_condition(fund_pool))
        data.append(hbdb.db2df(sql, db='funduser'))

    data = pd.concat(data, axis=0).reset_index(drop=True)

    data['month']=data['jsrq'].astype(str).str[4:6]
    for col in ['jgcybl','cyrs']:
        data[col+'_pct_change']=data.groupby('jjdm')[col].pct_change()
        data[col + '_pct_change_same_month'] = data.groupby(['jjdm','month'])[col].pct_change()
        data.loc[data[col+'_pct_change']==np.inf,col+'_pct_change']=1
        data.loc[data[col + '_pct_change_same_month'] == np.inf, col + '_pct_change_same_month'] = 1

    data.rename(columns={'jsrq': 'date'}, inplace=True)
    data.drop('month',axis=1,inplace=True)
    sql = "delete from factor_share "
    localdb.execute(sql, con=localdb)
    data.to_sql('factor_share', con=localdb, index=False, if_exists='append')

def save_sortino2db(rolling='2201'):

    sql= "SELECT distinct(tjrq) as tjrq from  st_fund.t_st_gm_zqjxxbl where tjrq>=20151231 "
    calander = hbdb.db2df(sql, db='funduser')
    calander['month_year']=calander['tjrq'].astype(str).str[0:6]
    calander.drop_duplicates('month_year',keep='last',inplace=True)
    calander['month']=calander['tjrq'].astype(str).str[4:6]
    calander=calander.loc[calander['month'].isin(['03','06','09','12'])]['tjrq'].tolist()
    # calander=calander['tjrq'].tolist()
    calander.sort()

    data=[]
    for date in calander:
        fund_pool = util.get_mutual_stock_funds(str(date), 2)
        sql="select jjdm,tjrq,zbnp as sortino from st_fund.t_st_gm_zqjstn where zblb='{2}' and tjrq={0} and jjdm in ({1}) and zbnp!=99999 "\
            .format(int(date),util.list_sql_condition(fund_pool),rolling)
        data.append(hbdb.db2df(sql,db='funduser'))

    data=pd.concat(data,axis=0).reset_index(drop=True)
    data.rename(columns={'tjrq':'date'},inplace=True)
    data['zblb']=rolling

    # sql = "delete from factor_sortino where zblb='{}'".format(rolling)
    # localdb.execute(sql, con=localdb)
    data.to_sql('factor_sortino', con=localdb, index=False, if_exists='append')

def save_manager_data2db(rolling='2201'):

    sql= "SELECT distinct(tjrq) as tjrq from  st_fund.t_st_gm_zqjxxbl where tjrq>=20151231 "
    calander = hbdb.db2df(sql, db='funduser')
    calander['month_year']=calander['tjrq'].astype(str).str[0:6]
    calander.drop_duplicates('month_year',keep='last',inplace=True)
    calander['month']=calander['tjrq'].astype(str).str[4:6]
    calander=calander.loc[calander['month'].isin(['03','06','09','12'])]['tjrq'].tolist()
    # calander=calander['tjrq'].tolist()
    calander.sort()

    data=[]
    for date in calander:

        fund_pool = util.get_mutual_stock_funds(str(date), 2)
        sql="select rydm,dbjj from st_fund.t_st_gm_jlzgcptj where jsrq>='{0}' and jsrq<='{1}' and dbjj in ({2})"\
            .format(str(date)[0:6]+'01',str(date)[0:6]+'31',util.list_sql_condition(fund_pool))
        fund_manager_map=hbdb.db2df(sql,db='funduser')

        sql="select rydm,jzrq,zbnp as sharp from st_fund.t_st_gm_jlzqjxpbl where zblb='{2}' and jzrq>='{0}' and jzrq<='{1}' and rydm in ({3}) and zbnp!=99999 and zslx=1"\
            .format(str(date)[0:6]+'15',str(date)[0:6]+'31',rolling,util.list_sql_condition(fund_manager_map['rydm'].tolist()))
        sharp=hbdb.db2df(sql,db='funduser').sort_values('jzrq').drop_duplicates('rydm',keep='last')

        sql="select rydm,jzrq,zbnp as sortino from st_fund.t_st_gm_jlzqjstn where zblb='{2}' and jzrq>='{0}' and jzrq<='{1}' and rydm in ({3}) and zbnp!=99999 and zslx=1"\
            .format(str(date)[0:6]+'15',str(date)[0:6]+'31',rolling,util.list_sql_condition(fund_manager_map['rydm'].tolist()))
        sortino=hbdb.db2df(sql,db='funduser').sort_values('jzrq').drop_duplicates('rydm',keep='last')

        sql="select rydm,jzrq,zbnp as calmar from st_fund.t_st_gm_jlqjkmbl where zblb='{2}' and jzrq>='{0}' and jzrq<='{1}' and rydm in ({3}) and zbnp!=99999 and zslx=1"\
            .format(str(date)[0:6]+'15',str(date)[0:6]+'31',rolling,util.list_sql_condition(fund_manager_map['rydm'].tolist()))
        calmar=hbdb.db2df(sql,db='funduser').sort_values('jzrq').drop_duplicates('rydm',keep='last')

        sql="select rydm,jzrq,zbnp as annual_ret from st_fund.t_st_gm_jlrqjhb where zblb='{2}' and jzrq>='{0}' and jzrq<='{1}' and rydm in ({3}) and zbnp!=99999 and zslx=1"\
            .format(str(date)[0:6]+'15',str(date)[0:6]+'31',rolling,util.list_sql_condition(fund_manager_map['rydm'].tolist()))
        annual_ret=hbdb.db2df(sql,db='funduser').sort_values('jzrq').drop_duplicates('rydm',keep='last')


        manager_df=pd.merge(fund_manager_map,sharp,how='inner',on='rydm')
        manager_df=pd.merge(manager_df,sortino,how='outer',on=['rydm','jzrq']).fillna(0)
        manager_df=pd.merge(manager_df,calmar,how='outer',on=['rydm','jzrq']).fillna(0)
        manager_df=pd.merge(manager_df,annual_ret,how='outer',on=['rydm','jzrq']).fillna(0)

        data.append(manager_df)

    data=pd.concat(data,axis=0).reset_index(drop=True)
    data.rename(columns={'jzrq':'date'},inplace=True)
    data['zblb']=rolling

    # sql = "delete from factor_sortino where zblb='{}'".format(rolling)
    # localdb.execute(sql, con=localdb)
    data.to_sql('factor_manager_data', con=localdb, index=False, if_exists='append')

class For_outsider:

    def __init__(self):

        from sqlalchemy import create_engine
        sql_info={'Sql_ip': '192.168.223.152',
                     'Sql_user': 'admin',
                     'Sql_pass': 'mysql',
                     'Database': 'fe_data_outsider',
                     'port': '3306'}

        #set up the engine
        self.localdb= create_engine("mysql+pymysql://{}:{}@{}:{}/{}".format(
            sql_info['Sql_user'], sql_info['Sql_pass'],
            sql_info['Sql_ip'], sql_info['port'], sql_info['Database']))

    @staticmethod
    def get_stock_fund(asofdate):

        sql="select jjdm,jjmc,clrq  from st_fund.t_st_gm_jjxx where cpfl='2' and (ejfl='13' or ejfl='35' or ejfl='37') and jjzt='0' and clrq<='{0}'  ".format(asofdate)
        stock_jjdm=hbdb.db2df(sql,db='funduser').sort_values(['clrq','jjdm'])
        stock_jjdm.drop_duplicates(['jjmc'],keep='first',inplace=True)

        return  stock_jjdm['jjdm'].unique().tolist()

    @staticmethod
    def get_bond_fund(asofdate):

        sql="select jjdm,jjmc,clrq from st_fund.t_st_gm_jjxx where cpfl='2' and (ejfl='22' or ejfl='23' or ejfl='24' or ejfl='27' or ejfl='28') and jjzt='0' and clrq<='{0}'  "\
            .format(asofdate)
        bond_jjdm=hbdb.db2df(sql,db='funduser').sort_values(['clrq','jjdm'])
        bond_jjdm.drop_duplicates(['jjmc'],keep='first',inplace=True)

        return  bond_jjdm['jjdm'].unique().tolist()

    def save_monthly_features_factors2db(self,jjdm_list):

        #read alpha wind data from local excel
        alpha=pd.read_excel(r"E:\GitFolder\docs\机构用数据\wind_data.xlsx",sheet_name='alpha').T
        alpha.columns=alpha.iloc[0]
        alpha=alpha.iloc[2:-1]
        alpha.rename(columns={'基金代码':'date'},inplace=True)
        alpha['date']=alpha['date'].astype(str).str[0:10].str.replace('-','')
        alpha=alpha.set_index('date').T

        #read beta wind data from local excel
        beta=pd.read_excel(r"E:\GitFolder\docs\机构用数据\wind_data.xlsx",sheet_name='beta').T
        beta.columns=beta.iloc[0]
        beta=beta.iloc[2:-1]
        beta.rename(columns={'基金代码':'date'},inplace=True)
        beta['date']=beta['date'].astype(str).str[0:10].str.replace('-','')
        beta=beta.set_index('date').T

        #read alpha wind data from local excel
        timing=pd.read_excel(r"E:\GitFolder\docs\机构用数据\wind_data.xlsx",sheet_name='选时能力').T
        timing.columns=timing.iloc[0]
        timing=timing.iloc[2:-1]
        timing.rename(columns={'基金代码':'date'},inplace=True)
        timing['date']=timing['date'].astype(str).str[0:10].str.replace('-','')
        timing=timing.set_index('date').T


        date_list=timing.columns

        for date in date_list:

            sql="select jjdm,hb1y,rqzh as date from st_fund.t_st_gm_yhb where tjyf ='{0}' and jjdm in ({1}) and hb1y!=99999"\
                .format(date[0:6]
            ,util.list_sql_condition(jjdm_list))

            month_ret=hbdb.db2df(sql,db='funduser')
            monthly_factors=month_ret.copy()
            month_end=str(month_ret['date'].iloc[0])

            sql="select jjdm,zbnp as sharp_ratio,tjrq as date from st_fund.t_st_gm_zqjxpbl where tjrq >='{0}' and tjrq<='{1}' and jjdm in ({2}) and zblb='2101' and zbnp!=99999"\
                .format(month_end[0:6]+'20',month_end[0:6]+'31'
                        ,util.list_sql_condition(jjdm_list))
            sharp_ratio=hbdb.db2df(sql,db='funduser').drop_duplicates('jjdm',keep='last')
            monthly_factors=pd.merge(monthly_factors,sharp_ratio.drop('date',axis=1),how='outer',on='jjdm')

            sql="select jjdm,zbnp as vol,tjrq as date from st_fund.t_st_gm_zqjbdl where tjrq >='{0}' and tjrq<='{1}' and jjdm in ({2}) and zblb='2101' and zbnp!=99999"\
                .format(month_end[0:6]+'20',month_end[0:6]+'31'
                        ,util.list_sql_condition(jjdm_list))
            std=hbdb.db2df(sql,db='funduser').drop_duplicates('jjdm',keep='last')
            monthly_factors=pd.merge(monthly_factors,std.drop('date',axis=1),how='outer',on='jjdm')

            sql="select jjdm,zbnp as max_db,jzrq as date from st_fund.t_st_gm_rqjzdhc where jzrq >='{0}' and jzrq<='{1}' and jjdm in ({2}) and zblb='2101' and zbnp!=99999"\
                .format(month_end[0:6]+'20',month_end[0:6]+'31'
                        ,util.list_sql_condition(jjdm_list))
            max_drawback=hbdb.db2df(sql,db='funduser').drop_duplicates('jjdm',keep='last')
            monthly_factors=pd.merge(monthly_factors,max_drawback.drop('date',axis=1),how='outer',on='jjdm')

            sql="select jjdm,zbnp as sortino,tjrq as date from st_fund.t_st_gm_zqjstn where tjrq >='{0}' and tjrq <='{1}' and  jjdm in ({2}) and zblb='2101' and zbnp!=99999"\
                .format(month_end[0:6]+'20',month_end[0:6]+'31',
                        util.list_sql_condition(jjdm_list))
            sortino=hbdb.db2df(sql,db='funduser').drop_duplicates('jjdm',keep='last')
            monthly_factors=pd.merge(monthly_factors,sortino.drop('date',axis=1),how='outer',on='jjdm')

            sql = "select jjdm,zbnp as Calmar,jzrq as date from st_fund.t_st_gm_rqjkmbl where jzrq >='{0}' and jzrq<='{1}' and jjdm in ({2}) and zblb='2101' and zbnp!=99999" \
                .format(month_end[0:6]+'20',month_end[0:6]+'31'
                        , util.list_sql_condition(jjdm_list))
            calmar = hbdb.db2df(sql, db='funduser').drop_duplicates('jjdm',keep='last')
            monthly_factors=pd.merge(monthly_factors,calmar.drop('date',axis=1),how='outer',on='jjdm')

            sql = "select jjdm,zbnp as annual_ret,jzrq as date from st_fund.t_st_gm_rqjhb where jzrq >='{0}' and jzrq<='{1}' and jjdm in ({2}) and zblb='2201' and zbnp!=99999 " \
                .format(month_end[0:6]+'20',month_end[0:6]+'31', util.list_sql_condition(jjdm_list))
            annual_ret = hbdb.db2df(sql, db='funduser').drop_duplicates('jjdm',keep='last')
            monthly_factors=pd.merge(monthly_factors,annual_ret.drop('date',axis=1),how='outer',on='jjdm')

            alpha_temp=alpha[date].to_frame('alpha')
            alpha_temp['jjdm'] = alpha_temp.index.astype(str).str[0:6]

            beta_temp=beta[date].to_frame('beta')
            beta_temp['jjdm'] = beta_temp.index.astype(str).str[0:6]

            timing_temp=timing[date].to_frame('timing')
            timing_temp['jjdm'] = timing_temp.index.astype(str).str[0:6]

            monthly_factors=pd.merge(monthly_factors,alpha_temp,how='outer',on='jjdm')
            monthly_factors=pd.merge(monthly_factors,beta_temp,how='outer',on='jjdm')
            monthly_factors=pd.merge(monthly_factors,timing_temp,how='outer',on='jjdm')

            monthly_factors['date']=date


            self.localdb.execute("delete from monthly_factors_zy where date='{0}'".format(date))
            (monthly_factors.fillna(0)).to_sql('monthly_factors_zy',con=self.localdb,index=False,if_exists='append')
            print(date+" Done")

    def save_quarterly_features_factors2db(self,jjdm_list):

        date_list=pd.read_sql("select distinct(date) as date from monthly_factors_zy"
                              ,con=self.localdb)['date'].tolist()
        date_list.sort()
        date_list=pd.DataFrame(data=date_list,columns=['date'])
        date_list['month']=date_list['date'].astype(str).str[4:6]
        date_list=date_list[date_list['month'].isin(['03','06','09','12'])]

        for date in date_list['date']:

            sql="select jjzzc,jjdm from st_fund.t_st_gm_zcpz where jjdm in ({0}) and jsrq>='{1}' and jsrq<='{2}' "\
                .format(util.list_sql_condition(jjdm_list),date[0:6]+'01',date[0:6]+'31')
            size_factro=hbdb.db2df(sql,db='funduser').drop_duplicates('jjdm',keep='last')

            sql="select jjdm,data_value as liquidity from st_fund.r_st_nav_attr_df where style_factor='liquidity' and data_type='exposure' and jjdm in ({0}) and tjrq>='{1}' and tjrq<='{2}'"\
                .format(util.list_sql_condition(jjdm_list),date[0:6]+'01',date[0:6]+'31')
            vol_factor=hbdb.db2df(sql,db='funduser').drop_duplicates('jjdm',keep='last')

            quarterly_factors=pd.merge(size_factro,vol_factor,how='outer',on='jjdm').fillna(0)
            quarterly_factors['date']=date

            # self.localdb.execute("delete from monthly_factors_zy where date='{0}'".format(date))
            quarterly_factors.to_sql('quarterly_factors_zy',con=self.localdb,index=False,if_exists='append')
            print(date+" Done")





if __name__ == '__main__':


    # save_manager_data2db('2201')
    # save_manager_data2db('2106')

    # fo=For_outsider()
    #
    # fo.save_monthly_features_factors2db(fo.get_stock_fund('20220801')
    #                                    +fo.get_bond_fund('20220801'))

    # fo.save_quarterly_features_factors2db(fo.get_stock_fund('20220801')
    #                                    +fo.get_bond_fund('20220801'))

    #save_fund_rank_factor2db()

    #save_share_change2db()

    # save_sortino2db(rolling='2201')
    # save_sortino2db(rolling='2202')
    # save_sortino2db(rolling='2106')

    # save_informationratio2db('2106')
    # save_informationratio2db('2103')
    # save_informationratio2db('2101')
    save_informationratio2db('2201')
    # save_informationratio2db('2202')

    #
    #save_fund_size2db()

    #save_fund_return_factor2db()

    #save_fund_return_factor2db()

    # df=read_factors('factor_hsl')
    # df=df.groupby('jjdm').min()
    # df=df[df['hsl'] >= 350]
    #
    # jjdm_list=df.index.tolist()
    # jjdm_con=util.list_sql_condition(jjdm_list)
    #
    # sql="""select jjdm,hb1n,rqzh from st_fund.t_st_gm_nhb
    # where hb1n!=99999 and jjdm in ({0}) and tjnf in ('2015','2016','2017','2018','2019','2020','2021')"""\
    #     .format(jjdm_con)
    # ret=hbdb.db2df(sql,db='funduser')
    # ret=ret.groupby('jjdm')['hb1n'].apply(bhar)
    # ret.to_csv('hlsret.csv',encoding='gbk')


    print('')